@@ -205,14 +205,14 @@ struct wc_Sha512 {
 #ifdef WOLFSSL_SHA512
 
 
-WOLFSSL_API int wc_InitSha512(wc_Sha512*);
-WOLFSSL_API int wc_InitSha512_ex(wc_Sha512*, void*, int);
-WOLFSSL_API int wc_Sha512Update(wc_Sha512*, const byte*, word32);
-WOLFSSL_API int wc_Sha512FinalRaw(wc_Sha512*, byte*);
-WOLFSSL_API int wc_Sha512Final(wc_Sha512*, byte*);
-WOLFSSL_API void wc_Sha512Free(wc_Sha512*);
+WOLFSSL_API int wc_InitSha512(wc_Sha512* sha);
+WOLFSSL_API int wc_InitSha512_ex(wc_Sha512* sha, void* heap, int devId);
+WOLFSSL_API int wc_Sha512Update(wc_Sha512* sha, const byte* data, word32 len);
+WOLFSSL_API int wc_Sha512FinalRaw(wc_Sha512* sha512, byte* hash);
+WOLFSSL_API int wc_Sha512Final(wc_Sha512* sha512, byte* hash);
+WOLFSSL_API void wc_Sha512Free(wc_Sha512* sha);
 
-WOLFSSL_API int wc_Sha512GetHash(wc_Sha512*, byte*);
+WOLFSSL_API int wc_Sha512GetHash(wc_Sha512* sha512, byte* hash);
 WOLFSSL_API int wc_Sha512Copy(wc_Sha512* src, wc_Sha512* dst);
 
 #ifdef WOLFSSL_HASH_FLAGS
@@ -225,13 +225,13 @@ WOLFSSL_API int wc_Sha512Transform(wc_Sha512* sha, const unsigned char* data);
 #endif
 
 #if !defined(WOLFSSL_NOSHA512_224)
-WOLFSSL_API int wc_InitSha512_224(wc_Sha512*);
-WOLFSSL_API int wc_InitSha512_224_ex(wc_Sha512*, void*, int);
-WOLFSSL_API int wc_Sha512_224Update(wc_Sha512*, const byte*, word32);
-WOLFSSL_API int wc_Sha512_224FinalRaw(wc_Sha512*, byte*);
-WOLFSSL_API int wc_Sha512_224Final(wc_Sha512*, byte*);
-WOLFSSL_API void wc_Sha512_224Free(wc_Sha512*);
-WOLFSSL_API int wc_Sha512_224GetHash(wc_Sha512*, byte*);
+WOLFSSL_API int wc_InitSha512_224(wc_Sha512* sha);
+WOLFSSL_API int wc_InitSha512_224_ex(wc_Sha512* sha, void* heap, int devId);
+WOLFSSL_API int wc_Sha512_224Update(wc_Sha512* sha, const byte* data, word32 len);
+WOLFSSL_API int wc_Sha512_224FinalRaw(wc_Sha512* sha512, byte* hash);
+WOLFSSL_API int wc_Sha512_224Final(wc_Sha512* sha512, byte* hash);
+WOLFSSL_API void wc_Sha512_224Free(wc_Sha512* sha);
+WOLFSSL_API int wc_Sha512_224GetHash(wc_Sha512* sha512, byte* hash);
 WOLFSSL_API int wc_Sha512_224Copy(wc_Sha512* src, wc_Sha512* dst);
 #ifdef WOLFSSL_HASH_FLAGS
     WOLFSSL_API int wc_Sha512_224SetFlags(wc_Sha512* sha512, word32 flags);
@@ -245,13 +245,13 @@ WOLFSSL_API int wc_Sha512_224Transform(wc_Sha512* sha,
 #endif /* !WOLFSSL_NOSHA512_224 */
 
 #if !defined(WOLFSSL_NOSHA512_256)
-WOLFSSL_API int wc_InitSha512_256(wc_Sha512*);
-WOLFSSL_API int wc_InitSha512_256_ex(wc_Sha512*, void*, int);
-WOLFSSL_API int wc_Sha512_256Update(wc_Sha512*, const byte*, word32);
-WOLFSSL_API int wc_Sha512_256FinalRaw(wc_Sha512*, byte*);
-WOLFSSL_API int wc_Sha512_256Final(wc_Sha512*, byte*);
-WOLFSSL_API void wc_Sha512_256Free(wc_Sha512*);
-WOLFSSL_API int wc_Sha512_256GetHash(wc_Sha512*, byte*);
+WOLFSSL_API int wc_InitSha512_256(wc_Sha512* sha);
+WOLFSSL_API int wc_InitSha512_256_ex(wc_Sha512* sha, void* heap, int devId);
+WOLFSSL_API int wc_Sha512_256Update(wc_Sha512* sha, const byte* data, word32 len);
+WOLFSSL_API int wc_Sha512_256FinalRaw(wc_Sha512* sha512, byte* hash);
+WOLFSSL_API int wc_Sha512_256Final(wc_Sha512* sha512, byte* hash);
+WOLFSSL_API void wc_Sha512_256Free(wc_Sha512* sha);
+WOLFSSL_API int wc_Sha512_256GetHash(wc_Sha512* sha512, byte* hash);
 WOLFSSL_API int wc_Sha512_256Copy(wc_Sha512* src, wc_Sha512* dst);
 #ifdef WOLFSSL_HASH_FLAGS
     WOLFSSL_API int wc_Sha512_256SetFlags(wc_Sha512* sha512, word32 flags);
@@ -300,14 +300,14 @@ enum {
 #endif
 #endif /* HAVE_FIPS */
 
-WOLFSSL_API int wc_InitSha384(wc_Sha384*);
-WOLFSSL_API int wc_InitSha384_ex(wc_Sha384*, void*, int);
-WOLFSSL_API int wc_Sha384Update(wc_Sha384*, const byte*, word32);
-WOLFSSL_API int wc_Sha384FinalRaw(wc_Sha384*, byte*);
-WOLFSSL_API int wc_Sha384Final(wc_Sha384*, byte*);
-WOLFSSL_API void wc_Sha384Free(wc_Sha384*);
+WOLFSSL_API int wc_InitSha384(wc_Sha384* sha);
+WOLFSSL_API int wc_InitSha384_ex(wc_Sha384* sha, void* heap, int devId);
+WOLFSSL_API int wc_Sha384Update(wc_Sha384* sha, const byte* data, word32 len);
+WOLFSSL_API int wc_Sha384FinalRaw(wc_Sha384* sha384, byte* hash);
+WOLFSSL_API int wc_Sha384Final(wc_Sha384* sha384, byte* hash);
+WOLFSSL_API void wc_Sha384Free(wc_Sha384* sha);
 
-WOLFSSL_API int wc_Sha384GetHash(wc_Sha384*, byte*);
+WOLFSSL_API int wc_Sha384GetHash(wc_Sha384* sha384, byte* hash);
 WOLFSSL_API int wc_Sha384Copy(wc_Sha384* src, wc_Sha384* dst);
 
 #ifdef WOLFSSL_HASH_FLAGS
