@@ -33,21 +33,20 @@
 */
 namespace ha_mcs_impl
 {
-inline void log_this(THD *thd, const char *message,
-                     logging::LOG_TYPE log_type, unsigned sid)
+inline void log_this(THD* thd, const char* message, logging::LOG_TYPE log_type, unsigned sid)
 {
-    // corresponds with dbcon in SubsystemID vector
-    // in messagelog.cpp
-    unsigned int subSystemId = 24;
-    logging::LoggingID logid( subSystemId, sid, 0);
-    logging::Message::Args args1;
-    logging::Message msg(1);
-    args1.add(message);
-    msg.format( args1 );
-    logging::Logger logger(logid.fSubsysID);
-    logger.logMessage(log_type, msg, logid);
+  // corresponds with dbcon in SubsystemID vector
+  // in messagelog.cpp
+  unsigned int subSystemId = 24;
+  logging::LoggingID logid(subSystemId, sid, 0);
+  logging::Message::Args args1;
+  logging::Message msg(1);
+  args1.add(message);
+  msg.format(args1);
+  logging::Logger logger(logid.fSubsysID);
+  logger.logMessage(log_type, msg, logid);
 }
 
-};
+};  // namespace ha_mcs_impl
 
 #endif
