@@ -209,6 +209,18 @@ int my_strnncollsp_simple(CHARSET_INFO * cs, const uchar *a, size_t a_length,
 }
 
 
+static int
+my_strnncollsp_nchars_simple(CHARSET_INFO * cs,
+                             const uchar *a, size_t a_length,
+                             const uchar *b, size_t b_length,
+                             size_t nchars)
+{
+  set_if_smaller(a_length, nchars);
+  set_if_smaller(b_length, nchars);
+  return my_strnncollsp_simple(cs, a, a_length, b, b_length);
+}
+
+
 int my_strnncollsp_simple_nopad(CHARSET_INFO * cs,
                                 const uchar *a, size_t a_length,
                                 const uchar *b, size_t b_length)
@@ -2127,6 +2139,7 @@ MY_COLLATION_HANDLER my_collation_8bit_simple_ci_handler =
     my_coll_init_simple,	/* init */
     my_strnncoll_simple,
     my_strnncollsp_simple,
+    my_strnncollsp_nchars_simple,
     my_strnxfrm_simple,
     my_strnxfrmlen_simple,
     my_like_range_simple,
@@ -2145,6 +2158,7 @@ MY_COLLATION_HANDLER my_collation_8bit_simple_nopad_ci_handler =
     my_coll_init_simple,	/* init */
     my_strnncoll_simple,
     my_strnncollsp_simple_nopad,
+    my_strnncollsp_nchars_simple,
     my_strnxfrm_simple_nopad,
     my_strnxfrmlen_simple,
     my_like_range_simple,
